@@ -1,6 +1,6 @@
 # Vend::Table::DBI - Access a table stored in an DBI/DBD database
 #
-# $Id: DBI_CompositeKey.pm,v 1.14 2008-05-18 02:50:21 jon Exp $
+# $Id: DBI_CompositeKey.pm,v 1.15 2008-05-26 02:30:04 markj Exp $
 #
 # Copyright (C) 2002-2008 Interchange Development Group
 # Copyright (C) 1996-2002 Red Hat, Inc.
@@ -21,7 +21,7 @@
 # MA  02110-1301  USA.
 
 package Vend::Table::DBI_CompositeKey;
-$VERSION = substr(q$Revision: 1.14 $, 10);
+$VERSION = substr(q$Revision: 1.15 $, 10);
 
 use strict;
 
@@ -340,21 +340,27 @@ sub set_slice {
 		$fary = [@$fin];
 		$vary = [@$vin];
 	}
-	elsif (ref $fin eq 'HASH') {
-		my $href = { %$fin };
-
-		if(! $key) {
-			@key = ();
-			for( @{$s->[$CONFIG]{_Key_columns}} ) {
-				push @key, delete $href->{$_};
-			}
-			$key = \@key;
-			$exists = $s->record_exists(\@key);
+ 	else {
+		my $href = $fin;
+		if(ref $href eq 'HASH') {
+			$href = { %$href };
 		}
-
+		else {
+			$href = { splice (@_, 2) };
+		}
+ 
+ 		if(! $key) {
+ 			@key = ();
+ 			for( @{$s->[$CONFIG]{_Key_columns}} ) {
+				push @key, delete $href->{$_};
+ 			}
+ 			$key = \@key;
+ 			$exists = $s->record_exists(\@key);
+ 		}
+ 
 		$vary = [ values %$href ];
 		$fary = [ keys   %$href ];
-	}
+ 	}
 
 	if(! $key) {
 		for my $kp (@{$s->[$CONFIG]{_Key_columns}}) {
