@@ -50,6 +50,7 @@ unless( $ENV{MINIVEND_DISABLE_UTF8} ) {
 	generate_key
 	get_option_hash
 	hash_string
+	header_data_scrub
 	hexify
 	is_hash
 	is_no
@@ -1426,16 +1427,7 @@ sub check_authorization {
 						);
 	my $cmp_pw;
 	my $use_crypt = 1;
-	if(!defined $Vend::Cfg) {
-		$pwinfo = $Global::AdminUser;
-		$pwinfo =~ s/^\s+//;
-		$pwinfo =~ s/\s+$//;
-		my (%compare) = split /[\s:]+/, $pwinfo;
-		return undef unless $compare{$user};
-		$cmp_pw = $compare{$user};
-		undef $use_crypt if $Global::Variable->{MV_NO_CRYPT};
-	}
-	elsif(	$user eq $Vend::Cfg->{RemoteUser}	and
+	if(	$user eq $Vend::Cfg->{RemoteUser}	and
 			$Vend::Cfg->{Password}					)
 	{
 		$cmp_pw = $Vend::Cfg->{Password};
@@ -1715,6 +1707,44 @@ sub show_times {
 	logDebug("$message: " . join " ", @times);
 }
 
+# This %syslog_constant_map is an attempt to work around a strange problem
+# where the eval inside &Sys::Syslog::xlate fails, which then croaks.
+# The cause of this freakish problem is still to be determined.
+
+my %syslog_constant_map;
+
+sub setup_syslog_constant_map {
+	for (
+		(map { "local$_" } (0..7)),
+		qw(
+			auth
+			authpriv
+			cron
+			daemon
+			ftp
+			kern
+			lpr
+			mail
+			news
+			syslog
+			user
+			uucp
+
+			emerg
+			alert
+			crit
+			err
+			warning
+			notice
+			info
+			debug
+		)
+	) {
+		$syslog_constant_map{$_} = Sys::Syslog::xlate($_);
+	}
+	return;
+}
+
 sub logGlobal {
 	return 1 if $Vend::ExternalProgram;
 
@@ -1784,6 +1814,7 @@ sub logGlobal {
 					print "to report this error:\n", $msg;
 					exit 1;
 				}
+				setup_syslog_constant_map() unless %syslog_constant_map;
 				$Vend::SysLogReady = 1;
 			}
 		}
@@ -1827,10 +1858,15 @@ sub logGlobal {
 
 	}
 	elsif ($Vend::SysLogReady) {
-		# Do not call this from within an eval, because in some strange circumstances
-		# it will cause the eval inside &Sys::Syslog::xlate to fail, which then croaks.
-		# The cause of this freakish problem is still to be determined.
-		Sys::Syslog::syslog "$level|$facility", $msg;
+		eval {
+			# avoid eval in Sys::Syslog::xlate() by using cached constants where possible
+			my $level_mapped = $syslog_constant_map{$level};
+			$level_mapped = $level unless defined $level_mapped;
+			my $facility_mapped = $syslog_constant_map{$facility};
+			$facility_mapped = $facility unless defined $facility_mapped;
+			my $priority = "$level_mapped|$facility_mapped";
+			Sys::Syslog::syslog $priority, $msg;
+		};
 	}
 
 	return 1;
@@ -2449,6 +2485,16 @@ sub backtrace {
     undef;
 }
 
+sub header_data_scrub {
+	my ($head_data) = @_;
+
+	## "HTTP Response Splitting" Exploit Fix
+	## http://www.securiteam.com/securityreviews/5WP0E2KFGK.html
+	$head_data =~ s/(?:%0[da]|[\r\n]+)+//ig;
+
+	return $head_data;
+}
+
 ### Provide stubs for former Vend::Util functions relocated to Vend::File
 *canonpath = \&Vend::File::canonpath;
 *catdir = \&Vend::File::catdir;
